var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
var __async = (__this, __arguments, generator) => {
  return new Promise((resolve, reject) => {
    var fulfilled = (value) => {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    };
    var rejected = (value) => {
      try {
        step(generator.throw(value));
      } catch (e) {
        reject(e);
      }
    };
    var step = (x) => x.done ? resolve(x.value) : Promise.resolve(x.value).then(fulfilled, rejected);
    step((generator = generator.apply(__this, __arguments)).next());
  });
};

// src/lib/load-query.ts
import {
  parseSelectParam,
  extractPathsFromFilters
} from "@supabase-cache-helpers/postgrest-filter";

// src/lib/build-select-statement.ts
import {
  groupPathsRecursive,
  isNestedPath
} from "@supabase-cache-helpers/postgrest-filter";
var buildSelectStatement = (paths) => {
  return groupPathsRecursive(paths).map((i) => {
    if (isNestedPath(i)) {
      return `${i.declaration}(${buildSelectStatement(i.paths)})`;
    }
    return `${i.alias ? `${i.alias}:` : ""}${i.path}`;
  }).join(",");
};

// src/lib/remove-alias-from-declaration.ts
var removeAliasFromDeclaration = (d) => d.split(".").map((el) => el.split(":").pop()).join(".");

// src/lib/load-query.ts
var loadQuery = ({
  query,
  disabled,
  queriesForTable
}) => {
  var _a;
  const userQueryPaths = query ? parseSelectParam(query) : null;
  const paths = userQueryPaths ? userQueryPaths.map((q) => ({
    declaration: removeAliasFromDeclaration(q.declaration),
    path: q.path
  })) : [];
  if (!disabled) {
    for (const tableQuery of queriesForTable()) {
      for (const filterPath of extractPathsFromFilters(tableQuery.filters)) {
        const path = (_a = tableQuery.paths.find(
          (p) => p.path === filterPath.path && p.alias === filterPath.alias
        )) != null ? _a : __spreadProps(__spreadValues({}, filterPath), {
          declaration: filterPath.path
        });
        if (paths.every(
          (p) => removeAliasFromDeclaration(p.declaration) !== removeAliasFromDeclaration(path.declaration)
        )) {
          paths.push({
            path: path.path,
            declaration: path.declaration.split(":").pop()
          });
        }
      }
      for (const path of tableQuery.paths) {
        if (paths.every(
          (p) => removeAliasFromDeclaration(p.declaration) !== removeAliasFromDeclaration(path.declaration)
        ) && // do not add agg functions
        !path.declaration.endsWith(".count")) {
          paths.push({
            path: path.path,
            declaration: path.declaration.split(":").pop()
          });
        }
      }
    }
  }
  const selectQuery = buildSelectStatement(paths);
  if (selectQuery.length === 0)
    return null;
  return { selectQuery, userQueryPaths, paths };
};

// src/delete.ts
var buildDeleteFetcher = (qb, primaryKeys, opts) => (input) => __async(void 0, null, function* () {
  let filterBuilder = qb.delete();
  for (const key of primaryKeys) {
    const value = input[key];
    if (!value)
      throw new Error(`Missing value for primary key ${String(key)}`);
    filterBuilder = filterBuilder.eq(key, value);
  }
  const primaryKeysData = primaryKeys.reduce((prev, key) => {
    return __spreadProps(__spreadValues({}, prev), {
      [key]: input[key]
    });
  }, {});
  if (!opts.disabled && opts.query) {
    const { data } = yield filterBuilder.select(opts.query).throwOnError().single();
    return {
      // since we are deleting, only the primary keys are required
      normalizedData: primaryKeysData,
      userQueryData: data
    };
  }
  yield filterBuilder.throwOnError().single();
  if (opts.queriesForTable().length > 0) {
    return { normalizedData: primaryKeysData };
  }
  return null;
});

// src/fetcher.ts
import {
  isPostgrestBuilder
} from "@supabase-cache-helpers/postgrest-shared";
var fetcher = (q) => __async(void 0, null, function* () {
  if (isPostgrestBuilder(q)) {
    q = q.throwOnError();
  }
  return yield q;
});

// src/lib/mutation-response.ts
import { transformRecursive } from "@supabase-cache-helpers/postgrest-filter";
var buildMutationFetcherResponse = (input, { paths, userQueryPaths }) => ({
  normalizedData: transformRecursive(paths, input, "path"),
  userQueryData: userQueryPaths ? transformRecursive(userQueryPaths, input, "alias") : void 0
});

// src/insert.ts
function buildInsertFetcher(qb, opts) {
  return (input) => __async(this, null, function* () {
    const query = loadQuery(opts);
    if (query) {
      const { selectQuery, userQueryPaths, paths } = query;
      const { data } = yield qb.insert(input).select(selectQuery).throwOnError();
      return data.map(
        (d) => buildMutationFetcherResponse(d, { paths, userQueryPaths })
      );
    }
    yield qb.insert(input).throwOnError();
    return null;
  });
}

// src/pagination-fetcher.ts
var createPaginationFetcher = (query, decode, pageSize) => {
  if (!query)
    return null;
  return (args) => __async(void 0, null, function* () {
    var _a, _b;
    const decodedKey = decode(args);
    const limit = ((_a = decodedKey.limit) != null ? _a : pageSize) - 1;
    const offset = (_b = decodedKey.offset) != null ? _b : 0;
    const { data } = yield query.range(offset, offset + limit).throwOnError();
    return data;
  });
};
var createPaginationHasMoreFetcher = (query, decode, pageSize) => {
  if (!query)
    return null;
  return (args) => __async(void 0, null, function* () {
    var _a, _b;
    const decodedKey = decode(args);
    const limit = (_a = decodedKey.limit) != null ? _a : pageSize;
    const offset = (_b = decodedKey.offset) != null ? _b : 0;
    const { data } = yield query.range(offset, offset + limit).throwOnError();
    let hasMore = false;
    if (data && data.length === pageSize + 1) {
      hasMore = true;
      data.pop();
    }
    return {
      // cannot be null because of .throwOnError()
      data,
      hasMore
    };
  });
};

// src/update.ts
var buildUpdateFetcher = (qb, primaryKeys, opts) => (input) => __async(void 0, null, function* () {
  let filterBuilder = qb.update(input);
  for (const key of primaryKeys) {
    const value = input[key];
    if (!value)
      throw new Error(`Missing value for primary key ${String(key)}`);
    filterBuilder = filterBuilder.eq(key, value);
  }
  const query = loadQuery(opts);
  if (query) {
    const { selectQuery, userQueryPaths, paths } = query;
    const { data } = yield filterBuilder.select(selectQuery).throwOnError().single();
    return buildMutationFetcherResponse(data, { userQueryPaths, paths });
  }
  yield filterBuilder.throwOnError().single();
  return null;
});

// src/upsert.ts
var buildUpsertFetcher = (qb, opts) => (input) => __async(void 0, null, function* () {
  const query = loadQuery(opts);
  if (query) {
    const { selectQuery, userQueryPaths, paths } = query;
    const { data } = yield qb.upsert(input).throwOnError().select(selectQuery);
    return data.map(
      (d) => buildMutationFetcherResponse(d, { paths, userQueryPaths })
    );
  }
  yield qb.upsert(input).throwOnError();
  return null;
});
export {
  buildDeleteFetcher,
  buildInsertFetcher,
  buildUpdateFetcher,
  buildUpsertFetcher,
  createPaginationFetcher,
  createPaginationHasMoreFetcher,
  fetcher,
  loadQuery
};
//# sourceMappingURL=index.mjs.map